/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package pl.edu.icm.yadda.analysis.packscanner;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Hashtable;
import java.util.List;
import morfologik.stemming.PolishStemmer;
import morfologik.stemming.WordData;
import pl.edu.icm.yadda.bwmeta.model.YConstants;
import pl.edu.icm.yadda.bwmeta.model.YDescription;
import pl.edu.icm.yadda.bwmeta.model.YElement;
import pl.edu.icm.yadda.bwmeta.model.YName;
import pl.edu.icm.yadda.bwmeta.model.YStructure;
import pl.edu.icm.yadda.parsing.other.Pair;

/**
 *
 * @author kura
 */
public class CountKeywordsStatistics {

    PolishStemmer stemmer = null;

    public static class KeyPhraseStats {

        public KeyPhraseStats(String lowerCased) {
            this.lowerCased = lowerCased;
        }
        ArrayList<String> stemmed = new ArrayList<String>();
        String lowerCased;
        int appears = 0;
        int appersInAbstract = 0;
        int appearsInAbstractOrContent = 0;
        int appearsInTitle = 0;
        int appearsInAbstractOrTitle = 0;
        int hasInAbstractOrContent = 0;
        int hasAbstact = 0;
    }
    int total = 0;
    int totalWithAbstact = 0;
    int totalWithContent = 0;
    int totalWithTitles = 0;
    int totalKeyowordsInstances = 0;
    int totalKeyowordsInstancesInAbstracts = 0;
    int totalKeyowordsInstancesInTitles = 0;
    int totalKeyowordsInstancesInTitlesOrAbstract = 0;
    Hashtable<String, KeyPhraseStats> stats = new Hashtable<String, KeyPhraseStats>();

    public String normalize(String toNormalize) {
        return toNormalize.toLowerCase().replaceAll("\\s+", " ").trim();
    }

    public ArrayList<String> allPossibleTexts(ArrayList<String>[] swords, int index) {
        ArrayList<String> ret = new ArrayList<String>();
        if (index == (swords.length - 1)) {
            return swords[index];
        }
        ArrayList<String> fall = allPossibleTexts(swords, index + 1);
        for (String s : swords[index]) {
            for (String f : fall) {
                ret.add(s + " " + f);
            }
        }
        return ret;
    }

    ArrayList<KeyPhraseStats> getKeyPhrases(YElement el) {
        if (el.getTagList(YConstants.TG_KEYWORD) == null) {
            return null;
        }
        ArrayList<KeyPhraseStats> ret = new ArrayList<KeyPhraseStats>();
        for (String s : el.getTagList(YConstants.TG_KEYWORD).getValues()) {
            String t = normalize(s);
            KeyPhraseStats stat = stats.get(t);
            if (stat == null) {
                stat = new KeyPhraseStats(t);
                stats.put(t, stat);
                stat.stemmed.add(t);
                if (stemmer != null) {
                    String[] words = t.split("\\s");
                    ArrayList<String>[] swords = new ArrayList[words.length];

                    for (int i = 0; i < words.length; i++) {
                        List<WordData> dat = stemmer.lookup(s);
                        swords[i] = new ArrayList<String>();
                        for (WordData d : dat) {
                            swords[i].add(d.getStem().toString());
                        }

                    }
                    for (String st : allPossibleTexts(swords, 0)) {
                        if (!stat.stemmed.contains(st)) {
                            stat.stemmed.add(st);
                        }
                    }
                }
            }
            ret.add(stat);
        }
        return ret;
    }

    void printInfo(PrintStream ps) {
        ps.println("Got Informations about :" + total + " elements");
        ps.println(totalWithAbstact + " elements has abstracts");
        ps.println(totalWithTitles + " elements has titles");
        ps.println("was found diffrent: " + stats.values().size() + " keyphrases");
        ps.println("was found: " + totalKeyowordsInstances + " keywords instances");
        ps.println("and from them " + totalKeyowordsInstancesInAbstracts + " (" + (totalKeyowordsInstances > 0 ? (100 * totalKeyowordsInstancesInAbstracts / totalKeyowordsInstances) : 0) + " %) appared in abstracts");
        ps.println("and from them " + totalKeyowordsInstancesInTitles + " (" + (totalKeyowordsInstances > 0 ? (100 * totalKeyowordsInstancesInTitles / totalKeyowordsInstances) : 0) + " %) appared in title");
        ps.println("and from them " + totalKeyowordsInstancesInTitlesOrAbstract + " (" + (totalKeyowordsInstances > 0 ? (100 * totalKeyowordsInstancesInTitlesOrAbstract / totalKeyowordsInstances) : 0) + " %) appared in title or abstract");

        ps.println("Keywords statistics: ");
        ArrayList<KeyPhraseStats> list = new ArrayList<KeyPhraseStats>(stats.values());
        Collections.sort(list, new Comparator<KeyPhraseStats>() {

            @Override
            public int compare(KeyPhraseStats o1, KeyPhraseStats o2) {
                return o1.appears - o2.appears;
            }
        });
        ps.println("phrase;appears;has absttract;appers with abstract;was in % abstracts;appearsInTitles;was in % titles;appers in  title or abstract; was in % titles or abstracts");
        for (KeyPhraseStats stat : list) {
            ps.println(stat.lowerCased + ";" + stat.appears + ";" + stat.hasAbstact + ";" + stat.appersInAbstract + ";" + (stat.hasAbstact > 0 ? ((100 * stat.appersInAbstract) / stat.hasAbstact) : 0) + ";" + stat.appearsInTitle + ";" + (100 * stat.appearsInTitle) / stat.appears + ";" + stat.appearsInAbstractOrTitle + ";" + (100 * stat.appearsInAbstractOrTitle) / stat.appears);

        }


    }

    public static enum TextType {

        ABSTRACT,
        TITLE
    }

    public void processYElement(YElement elem) {
        //   ArrayList<String> texts;
        boolean article = false;
        YStructure stru = elem.getStructure(YConstants.EXT_HIERARCHY_JOURNAL);
        if (stru == null) {
            return;
        }
        if (!YConstants.EXT_LEVEL_JOURNAL_ARTICLE.equalsIgnoreCase(stru.getCurrent().getLevel())) {
            return;
        }


        ArrayList<Pair<String, TextType>> textsTexts = new ArrayList<Pair<String, TextType>>();
        ArrayList<KeyPhraseStats> keys = getKeyPhrases(elem);
        if (keys == null) {
            return;
        }
        int longestStemmedPhrase = 0;

        Hashtable<String, KeyPhraseStats> stemsToKeys = new Hashtable<String, KeyPhraseStats>();
        for (KeyPhraseStats s : keys) {
            stemsToKeys.put(s.lowerCased, s);
            int i = s.lowerCased.split("\\s").length;
            if (i > longestStemmedPhrase) {
                longestStemmedPhrase = i;
            }
            for (String stem : s.stemmed) {
                i = stem.split("\\s").length;
                if (i > longestStemmedPhrase) {
                    longestStemmedPhrase = i;
                }
                stemsToKeys.put(stem, s);
            }
        }

        ArrayList<String>[] lastStem = new ArrayList[longestStemmedPhrase];
        boolean[] inAbstracts = new boolean[keys.size()];
        boolean[] inTitles = new boolean[keys.size()];
        for (int i = 0; i < inAbstracts.length; i++) {
            inAbstracts[i] = false;
        }
        for (int i = 0; i < inAbstracts.length; i++) {
            inTitles[i] = false;
        }

        total++;
        boolean hasAbstarct = false;
        boolean hasTitle = false;
        for (YDescription desc : elem.getDescriptions()) {
            if (YConstants.DS_ABSTRACT.equalsIgnoreCase(desc.getType())) {
                textsTexts.add(new Pair<String, TextType>(normalize(desc.getText()), TextType.ABSTRACT));
                hasAbstarct = true;
            }
        }
        for (YName name : elem.getNames()) {
            textsTexts.add(new Pair<String, TextType>(normalize(name.getText()), TextType.TITLE));
            hasTitle = true;
        }

        for (Pair<String, TextType> te : textsTexts) {
            if (lastStem.length > 0 && stemmer != null) {
                String abs = te.getFirst().replaceAll("\\W", " ").replaceAll("\\s+", " ");
                String[] words = abs.split("\\s");
                for (int i = 0; i < lastStem.length; i++) {
                    lastStem[i] = null;
                }
                for (int i = 0; i < words.length; i++) {
                    List<WordData> dat = stemmer.lookup(words[i]);

                    ArrayList<String> li = new ArrayList<String>();
                    for (int j=0;j<dat.size();j++) {
                        li.add(dat.get(j).getStem().toString());
                    }
                    for (int j = 0; j < lastStem.length - 1; j++) {
                        lastStem[j] = lastStem[j + 1];
                    }
                    lastStem[lastStem.length - 1] = li;

                    int j = 0;
                    while (lastStem[j] == null) {
                        j++;
                    }

                    for (int k = j; k < lastStem.length; k++) {
                        for (String s : allPossibleTexts(lastStem, k)) {
                            if (stemsToKeys.containsKey(s)) {
                                KeyPhraseStats stat = stemsToKeys.get(s);
                                if (keys.contains(stat)) {
                                    // a powinna
                                    int pos = keys.indexOf(stat);
                                    switch (te.getSecond()) {
                                        case ABSTRACT:
                                            inAbstracts[pos] = true;
                                            break;
                                        case TITLE:
                                            inTitles[pos] = true;
                                            break;
                                    }
                                }
                            }
                        }


                    }

                }


            }
        }


        if (hasAbstarct) {
            totalWithAbstact++;
        }
        if (hasTitle) {
            totalWithTitles++;
        }
        for (int j = 0; j < keys.size(); j++) {
            KeyPhraseStats stat = keys.get(j);
            totalKeyowordsInstances++;
            stat.appears++;
            boolean inAbstract = false;
            boolean inTitle = false;
            for (Pair<String, TextType> te : textsTexts) {
                if (te.getFirst().contains(stat.lowerCased)) {
                    switch (te.getSecond()) {
                        case ABSTRACT:
                            inAbstract = true;
                            break;
                        case TITLE:
                            inTitle = true;
                            break;
                    }

                }
                for (String stemmed : stat.stemmed) {
                    if (te.getFirst().contains(stemmed)) {
                           switch (te.getSecond()) {
                                case ABSTRACT:
                                    inAbstract = true;
                                    break;
                                case TITLE:
                                    inTitle = true;
                                    break;
                            }

                        

                    }
                }

            }
            inAbstract = inAbstract || inAbstracts[j];
            inTitle = inTitle || inTitles[j];
            if (hasAbstarct) {
                stat.hasAbstact++;
            }

            if (inAbstract) {
                stat.appersInAbstract++;
                totalKeyowordsInstancesInAbstracts++;
            }
            if (inTitle) {
                stat.appearsInTitle++;
                totalKeyowordsInstancesInTitles++;
            }
            if (inTitle || inAbstract) {
                totalKeyowordsInstancesInTitlesOrAbstract++;
                stat.appearsInAbstractOrTitle++;
            }

        }

    }

    public static void main(String[] args) throws FileNotFoundException {
        if (args.length != 1) {
            System.err.println("One argument - directory with files");
        }
        CountKeywordsStatistics counter = new CountKeywordsStatistics();
        if (args.length > 1) {
            if ("dostem".equalsIgnoreCase(args[1])) {
                counter.stemmer = new PolishStemmer();
            }
        }
        PackDirToYElementIterator iter = new PackDirToYElementIterator(new File(args[0]));
        while (iter.hasNext()) {
            YElement el = iter.next();
            counter.processYElement(el);
        }
        PrintStream ps = new PrintStream(new FileOutputStream("key-stats-" + System.currentTimeMillis() + ".log"));
        counter.printInfo(ps);
        ps.close();
        System.exit(0);

    }
}
