/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package pl.edu.icm.yadda.analysis.packscanner;

import pl.edu.icm.yadda.common.utils.Pair;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Hashtable;
import java.util.List;
import morfologik.stemming.PolishStemmer;
import morfologik.stemming.WordData;
import pl.edu.icm.yadda.bwmeta.model.YConstants;
import pl.edu.icm.yadda.bwmeta.model.YDescription;
import pl.edu.icm.yadda.bwmeta.model.YElement;
import pl.edu.icm.yadda.bwmeta.model.YLanguage;
import pl.edu.icm.yadda.bwmeta.model.YName;
import pl.edu.icm.yadda.bwmeta.model.YStructure;

/**
 *
 * @author kura
 */
public class TesterAndClassifier {

    PolishStemmer stemmer = null;

    public static class KeyPhraseStats {

        public KeyPhraseStats(String lowerCased) {
            this.lowerCased = lowerCased;
        }
        ArrayList<String> stemmed = new ArrayList<String>();
        String lowerCased;
        int appears = 0;
        int appersInAbstract = 0;
        int appearsInAbstractOrContent = 0;
        int appearsInTitle = 0;
        int appearsInAbstractOrTitle = 0;
        int hasInAbstractOrContent = 0;
        int hasAbstact = 0;
    }
    int total = 0;
    int totalWithAbstact = 0;
    int totalWithContent = 0;
    int totalWithTitles = 0;
    int totalKeyowordsInstances = 0;
    int totalKeyowordsInstancesInAbstracts = 0;
    int totalKeyowordsInstancesInTitles = 0;
    int totalKeyowordsInstancesInTitlesOrAbstract = 0;
    Hashtable<String, KeyPhraseStats> stats = new Hashtable<String, KeyPhraseStats>();

    public String normalize(String toNormalize) {
        return toNormalize.toLowerCase().replaceAll("\\s+", " ").trim();
    }

    public ArrayList<String> allPossibleTexts(ArrayList<String>[] swords, int index) {
        ArrayList<String> ret = new ArrayList<String>();
        if (index == (swords.length - 1)) {
            return swords[index];
        }
        ArrayList<String> fall = allPossibleTexts(swords, index + 1);
        for (String s : swords[index]) {
            for (String f : fall) {
                ret.add(s + " " + f);
            }
        }
        return ret;
    }

    ArrayList<KeyPhraseStats> getKeyPhrasesFromElement(YElement el) {
        if (el.getTagList(YConstants.TG_KEYWORD) == null) {
            return null;
        }
        ArrayList<KeyPhraseStats> ret = new ArrayList<KeyPhraseStats>();
        for (String s : el.getTagList(YConstants.TG_KEYWORD).getValues()) {
            String t = normalize(s);
            KeyPhraseStats stat = stats.get(t);
            if (stat == null) {
                stat = new KeyPhraseStats(t);
                stats.put(t, stat);
                stat.stemmed.add(t);
                if (stemmer != null) {
                    String[] words = t.split("\\s");
                    ArrayList<String>[] swords = new ArrayList[words.length];

                    for (int i = 0; i < words.length; i++) {
                        List<WordData> dat = stemmer.lookup(s);
                        swords[i] = new ArrayList<String>();
                        for (WordData d : dat) {
                            swords[i].add(d.getStem().toString());
                        }

                    }
                    for (String st : allPossibleTexts(swords, 0)) {
                        if (!stat.stemmed.contains(st)) {
                            stat.stemmed.add(st);
                        }
                    }
                }
            }
            ret.add(stat);
        }
        return ret;
    }

    void printInfo(PrintStream ps) {
        ps.println("Got Informations about :" + total + " elements");
        ps.println(totalWithAbstact + " elements has abstracts");
        ps.println(totalWithTitles + " elements has titles");
        ps.println("was found diffrent: " + stats.values().size() + " keyphrases");
        ps.println("was found: " + totalKeyowordsInstances + " keywords instances");
        ps.println("and from them " + totalKeyowordsInstancesInAbstracts + " (" + (totalKeyowordsInstances > 0 ? (100 * totalKeyowordsInstancesInAbstracts / totalKeyowordsInstances) : 0) + " %) appared in abstracts");
        ps.println("and from them " + totalKeyowordsInstancesInTitles + " (" + (totalKeyowordsInstances > 0 ? (100 * totalKeyowordsInstancesInTitles / totalKeyowordsInstances) : 0) + " %) appared in title");
        ps.println("and from them " + totalKeyowordsInstancesInTitlesOrAbstract + " (" + (totalKeyowordsInstances > 0 ? (100 * totalKeyowordsInstancesInTitlesOrAbstract / totalKeyowordsInstances) : 0) + " %) appared in title or abstract");

        ps.println("Keywords statistics: ");
        ArrayList<KeyPhraseStats> list = new ArrayList<KeyPhraseStats>(stats.values());
        Collections.sort(list, new Comparator<KeyPhraseStats>() {

            @Override
            public int compare(KeyPhraseStats o1, KeyPhraseStats o2) {
                return o1.appears - o2.appears;
            }
        });
        ps.println("phrase;appears;has absttract;appers with abstract;was in % abstracts;appearsInTitles;was in % titles;appers in  title or abstract; was in % titles or abstracts");
        for (KeyPhraseStats stat : list) {
            ps.println(stat.lowerCased + ";" + stat.appears + ";" + stat.hasAbstact + ";" + stat.appersInAbstract + ";" + (stat.hasAbstact > 0 ? ((100 * stat.appersInAbstract) / stat.hasAbstact) : 0) + ";" + stat.appearsInTitle + ";" + (100 * stat.appearsInTitle) / stat.appears + ";" + stat.appearsInAbstractOrTitle + ";" + (100 * stat.appearsInAbstractOrTitle) / stat.appears);

        }


    }

    public static enum TextType {

        ABSTRACT,
        TITLE
    }
    Hashtable<String, Integer> wordsInAbstractsCount = new Hashtable<String, Integer>();
    Hashtable<String, Integer> keyprasesCount = new Hashtable<String, Integer>();
    int numArticles = 0;
    Hashtable<Pair<String, String>, Integer> keyphraseWordCount = new Hashtable<Pair<String, String>, Integer>();
    Hashtable<String, ArrayList<String>> wordsToPossibleKeyphrases = new Hashtable<String, ArrayList<String>>();

    public void processYElementAndLearn(YElement elem,YLanguage lang) {
        YStructure stru = elem.getStructure(YConstants.EXT_HIERARCHY_JOURNAL);
        if (stru == null) {
            return;
        }
        if (!YConstants.EXT_LEVEL_JOURNAL_ARTICLE.equalsIgnoreCase(stru.getCurrent().getLevel())) {
            return;
        }
        ArrayList<KeyPhraseStats> keys = getKeyPhrasesFromElement(elem);
        if (keys == null) {
            return;
        }
        boolean hasAbstarct = false;
        boolean hasTitle = false;
        ArrayList<Pair<String, TextType>> textsTexts = new ArrayList<Pair<String, TextType>>();
        for (YDescription desc : elem.getDescriptions()) {
            if (YConstants.DS_ABSTRACT.equalsIgnoreCase(desc.getType())) {
                if (lang==null || lang.equals(desc.getLanguage())) {
                   textsTexts.add(new Pair<String, TextType>(normalize(desc.getText()), TextType.ABSTRACT));
                   hasAbstarct = true;
                }
            }
        }
        for (YName name : elem.getNames()) {
            if (lang==null || lang.equals(name.getLanguage())) {
               textsTexts.add(new Pair<String, TextType>(normalize(name.getText()), TextType.TITLE));
               hasTitle = true;
            }
        }
        if (!hasAbstarct) {
            return;
        }
        ArrayList<String> foundStems = new ArrayList<String>();
        for (Pair<String, TextType> te : textsTexts) {
            String abs = te.getFirst().replaceAll("\\W", " ").replaceAll("\\s+", " ");
            String[] words = abs.split("\\s");
            for (int i = 0; i < words.length; i++) {
                if (stemmer != null) {

                    for (WordData w : stemmer.lookup(words[i])) {
                        if (!foundStems.contains(w.getStem().toString())) {
                            foundStems.add(w.getStem().toString());

                            if (!wordsToPossibleKeyphrases.containsKey(w.getStem().toString())) {
                                wordsToPossibleKeyphrases.put(w.getStem().toString(), new ArrayList<String>());
                            }
                        }
                    }
                } else {

                    if (!foundStems.contains(words[i])) {
                        foundStems.add(words[i]);

                        if (!wordsToPossibleKeyphrases.containsKey(words[i])) {
                            wordsToPossibleKeyphrases.put(words[i], new ArrayList<String>());
                        }
                    }
                }
            }
        }

        for (String s : foundStems) {
            if (wordsInAbstractsCount.containsKey(s)) {
                wordsInAbstractsCount.put(s, wordsInAbstractsCount.get(s) + 1);
            } else {
                wordsInAbstractsCount.put(s, 1);
            }
        }

        for (KeyPhraseStats kps : keys) {
            String sname = kps.lowerCased;
            if (keyprasesCount.containsKey(sname)) {
                keyprasesCount.put(sname, keyprasesCount.get(sname) + 1);
            } else {
                keyprasesCount.put(sname, 1);
            }
            for (String s : foundStems) {
                Pair<String, String> kwp = new Pair<String, String>(sname, s);
                if (keyphraseWordCount.containsKey(kwp)) {
                    keyphraseWordCount.put(kwp, keyphraseWordCount.get(kwp) + 1);
                } else {
                    keyphraseWordCount.put(kwp, 1);
                }
                if (!wordsToPossibleKeyphrases.get(s).contains(sname)) {
                    wordsToPossibleKeyphrases.get(s).add(sname);
                }



            }

        }
        numArticles++;


    }

    public void reduceTables(PrintStream ps) {
        ArrayList<String> wordsToRemove = new ArrayList<String>();
        for (String word : wordsInAbstractsCount.keySet()) {
            boolean wordNeeded = false;
            int wc = wordsInAbstractsCount.get(word);
            if (wc>1) {
            for (String key : wordsToPossibleKeyphrases.get(word)) {
                int kwc = keyphraseWordCount.get(new Pair<String, String>(key, word));
                int kc = keyprasesCount.get(key);
                if (Math.pow((((double) kc) / ((double) numArticles)) - (((double) kwc) / ((double) wc)), 2) > 0.00001) {
                    wordNeeded = true;
                }
            }
            }
            if (!wordNeeded) {
                wordsToRemove.add(word);
            }
        }
        for (String w : wordsToRemove) {
            wordsInAbstractsCount.remove(w);
            for (String key : wordsToPossibleKeyphrases.get(w)) {
                keyphraseWordCount.remove(new Pair<String, String>(key, w));
            }
            wordsToPossibleKeyphrases.remove(w);
        }
        Hashtable<String, ArrayList<String>> keypharseToWords = new Hashtable<String, ArrayList<String>>();
        for (Pair<String, String> pa : keyphraseWordCount.keySet()) {
            if (!keypharseToWords.containsKey(pa.getFirst())) {
                keypharseToWords.put(pa.getFirst(), new ArrayList<String>());
            }
            keypharseToWords.get(pa.getFirst()).add(pa.getSecond());
        }


        for (String key : keyprasesCount.keySet()) {
            final String keyf = key;
            ArrayList<String> st = keypharseToWords.get(key);
            if (st != null) {
                Collections.sort(st, new Comparator<String>() {

                    @Override
                    public int compare(String o1, String o2) {
                        double p1 = ((double) keyphraseWordCount.get(new Pair<String, String>(keyf, o1))) / wordsInAbstractsCount.get(o1);
                        double p2 = ((double) keyphraseWordCount.get(new Pair<String, String>(keyf, o2))) / wordsInAbstractsCount.get(o2);
                        if (p1 - p2 > 0) {
                            return -1;
                        }
                        if (p1 - p2 < 0) {
                            return 1;
                        }
                        return 0;
                    }
                });
                ps.println(keyf + " : " + keyprasesCount.get(keyf));
                for (String s : st) {
                    ps.println("\t" + s + "\t p= " + (((double) keyphraseWordCount.get(new Pair<String, String>(keyf, s))) / wordsInAbstractsCount.get(s))
                        + " \t|pair| = " + keyphraseWordCount.get(new Pair<String, String>(keyf, s)) + "\t|word| " + wordsInAbstractsCount.get(s));
                }
                ps.println();

            }


        }

    }

    public ArrayList<String> generateKeyWordsFor(YElement elem, PrintStream ps,YLanguage lang) {
        YStructure stru = elem.getStructure(YConstants.EXT_HIERARCHY_JOURNAL);
        if (stru == null) {
            return null;
        }
        if (!YConstants.EXT_LEVEL_JOURNAL_ARTICLE.equalsIgnoreCase(stru.getCurrent().getLevel())) {
            return null;
        }
        boolean hasAbstarct = false;
        boolean hasTitle = false;
        ArrayList<Pair<String, TextType>> textsTexts = new ArrayList<Pair<String, TextType>>();
        for (YDescription desc : elem.getDescriptions()) {
            if (YConstants.DS_ABSTRACT.equalsIgnoreCase(desc.getType())) {
                if (lang==null || lang.equals(desc.getLanguage())) {
                  textsTexts.add(new Pair<String, TextType>(normalize(desc.getText()), TextType.ABSTRACT));
                  hasAbstarct = true;
                }
            }
        }
        for (YName name : elem.getNames()) {
            if (lang==null || lang.equals(name.getLanguage())) {
              textsTexts.add(new Pair<String, TextType>(normalize(name.getText()), TextType.TITLE));
              hasTitle = true;
            }
        }
        if (!hasAbstarct) {
            return null;
        }
        ArrayList<String> foundStems = new ArrayList<String>();
        final Hashtable<String, Double> possibleKeyPhrases = new Hashtable<String, Double>();
        final Hashtable<String, Double> possibleKeyPhrases2 = new Hashtable<String, Double>();
        for (Pair<String, TextType> te : textsTexts) {
            String abs = te.getFirst().replaceAll("\\W", " ").replaceAll("\\s+", " ");
            String[] words = abs.split("\\s");
            for (int i = 0; i < words.length; i++) {
                    if (stemmer != null) {

                        for (WordData w : stemmer.lookup(words[i])) {


                            if (!foundStems.contains(w.getStem().toString())) {
                                if (wordsToPossibleKeyphrases.containsKey(w.getStem().toString())) {
                                    foundStems.add(w.getStem().toString());
                                    for (String k : wordsToPossibleKeyphrases.get(w.getStem().toString())) {
                                        if (!possibleKeyPhrases.containsKey(k)) {
                                            possibleKeyPhrases.put(k, 1.0);
                                            possibleKeyPhrases2.put(k, 1.0);
                                        }
                                    }

                                }
                            }
                        }
                } else {






                if (!foundStems.contains(words[i])) {
                    if (wordsToPossibleKeyphrases.containsKey(words[i])) {
                        foundStems.add(words[i]);
                        for (String k : wordsToPossibleKeyphrases.get(words[i])) {
                            if (!possibleKeyPhrases.containsKey(k)) {
                                possibleKeyPhrases.put(k, 1.0);
                                possibleKeyPhrases2.put(k, 1.0);
                            }
                        }
                    }
                }
                }
                
            }
        }
        for (String w : foundStems) {
            for (String k : possibleKeyPhrases.keySet()) {

                Integer kwcI = keyphraseWordCount.get(new Pair<String, String>(k, w));
                int kwc = 0;
                if (kwcI != null) {
                    kwc = kwcI;
                }
                double pw = ((double) kwc) / ((double) wordsInAbstractsCount.get(w));
                if (pw == 0.00) {
                    pw = 0.00001;
                }
                if (pw == 1.00) {
                    pw = 0.99999;
                }
                possibleKeyPhrases.put(k, possibleKeyPhrases.get(k) * (1.0 - pw));
                possibleKeyPhrases2.put(k, possibleKeyPhrases2.get(k) * (pw));
            }
        }
        ArrayList<String> typedKeys = new ArrayList<String>(possibleKeyPhrases.keySet());
        //uwaga ketyPtrases jest odwrócona
        Collections.sort(typedKeys, new Comparator<String>() {

            @Override
            public int compare(String o1, String o2) {
                double ret = -(1-possibleKeyPhrases.get(o1))/keyprasesCount.get(o1) + (1-possibleKeyPhrases.get(o2))/keyprasesCount.get(o2);
                if (ret < 0) {
                    return -1;
                }
                if (ret > 0) {
                    return 1;
                }
                return 0;
            }
        });
        for (YName name : elem.getNames()) {
            ps.println("Title: " + name.getText());
        }


        for (YDescription desc : elem.getDescriptions()) {
            if (YConstants.DS_ABSTRACT.equalsIgnoreCase(desc.getType())) {
                textsTexts.add(new Pair<String, TextType>(normalize(desc.getText()), TextType.ABSTRACT));
                hasAbstarct = true;
            }
        }


        ArrayList<KeyPhraseStats> keys = getKeyPhrasesFromElement(elem);
        ps.println("Original keyphrases: ");
        if (keys != null) {
            for (KeyPhraseStats ksp : keys) {
                ps.println("\t" + ksp.lowerCased);
            }
        }
        ArrayList<String> ret = new ArrayList<String>();
        ps.println("Found keyphrase (reversed propblility): ");
        int i = 0;
        while (i < 10 && i < typedKeys.size()) {
            ps.println("\t\t" + typedKeys.get(i) + " - " + ((1.0 - possibleKeyPhrases.get(typedKeys.get(i)))/keyprasesCount.get(typedKeys.get(i))));
            ret.add(typedKeys.get(i));
            i++;
        }
        ps.println("Found keyphrase (normal): ");
        i = 0;
        Collections.sort(typedKeys, new Comparator<String>() {

            @Override
            public int compare(String o1, String o2) {
                double ret = possibleKeyPhrases2.get(o1)/keyprasesCount.get(o1) - possibleKeyPhrases2.get(o2)/keyprasesCount.get(o2);
                if (ret < 0) {
                    return 1;
                }
                if (ret > 0) {
                    return -1;
                }
                return 0;
            }
        });
        while (i < 10 && i < typedKeys.size()) {
            ps.println("\t\t" + typedKeys.get(i) + " - " + (possibleKeyPhrases2.get(typedKeys.get(i))));
            i++;
        }
        return ret;

    }

    /* Kawałek kodu z którego w przysżłości będzimey pobierać elementy dotyczące stemmingu
    public void processYElement(YElement elem) {
    //   ArrayList<String> texts;
    boolean article = false;
    YStructure stru = elem.getStructure(YConstants.EXT_HIERARCHY_JOURNAL);
    if (stru == null) {
    return;
    }
    if (!YConstants.EXT_LEVEL_JOURNAL_ARTICLE.equalsIgnoreCase(stru.getCurrent().getLevel())) {
    return;
    }


    ArrayList<Pair<String, TextType>> textsTexts = new ArrayList<Pair<String, TextType>>();
    ArrayList<KeyPhraseStats> keys = getKeyPhrasesFromElement(elem);
    if (keys == null) {
    return;
    }
    int longestStemmedPhrase = 0;

    Hashtable<String, KeyPhraseStats> stemsToKeys = new Hashtable<String, KeyPhraseStats>();
    for (KeyPhraseStats s : keys) {
    stemsToKeys.put(s.lowerCased, s);
    int i = s.lowerCased.split("\\s").length;
    if (i > longestStemmedPhrase) {
    longestStemmedPhrase = i;
    }
    for (String stem : s.stemmed) {
    i = stem.split("\\s").length;
    if (i > longestStemmedPhrase) {
    longestStemmedPhrase = i;
    }
    stemsToKeys.put(stem, s);
    }
    }

    ArrayList<String>[] lastStem = new ArrayList[longestStemmedPhrase];
    boolean[] inAbstracts = new boolean[keys.size()];
    boolean[] inTitles = new boolean[keys.size()];
    for (int i = 0; i < inAbstracts.length; i++) {
    inAbstracts[i] = false;
    }
    for (int i = 0; i < inAbstracts.length; i++) {
    inTitles[i] = false;
    }

    total++;
    boolean hasAbstarct = false;
    boolean hasTitle = false;
    for (YDescription desc : elem.getDescriptions()) {
    if (YConstants.DS_ABSTRACT.equalsIgnoreCase(desc.getType())) {
    textsTexts.add(new Pair<String, TextType>(normalize(desc.getText()), TextType.ABSTRACT));
    hasAbstarct = true;
    }
    }
    for (YName name : elem.getNames()) {
    textsTexts.add(new Pair<String, TextType>(normalize(name.getText()), TextType.TITLE));
    hasTitle = true;
    }

    for (Pair<String, TextType> te : textsTexts) {
    if (lastStem.length > 0 && stemmer != null) {
    String abs = te.getFirst().replaceAll("\\W", " ").replaceAll("\\s+", " ");
    String[] words = abs.split("\\s");
    for (int i = 0; i < lastStem.length; i++) {
    lastStem[i] = null;
    }
    for (int i = 0; i < words.length; i++) {
    List<WordData> dat = stemmer.lookup(words[i]);

    ArrayList<String> li = new ArrayList<String>();
    for (int j=0;j<dat.size();j++) {
    li.add(dat.get(j).getStem().toString());
    }
    for (int j = 0; j < lastStem.length - 1; j++) {
    lastStem[j] = lastStem[j + 1];
    }
    lastStem[lastStem.length - 1] = li;

    int j = 0;
    while (lastStem[j] == null) {
    j++;
    }

    for (int k = j; k < lastStem.length; k++) {
    for (String s : allPossibleTexts(lastStem, k)) {
    if (stemsToKeys.containsKey(s)) {
    KeyPhraseStats stat = stemsToKeys.get(s);
    if (keys.contains(stat)) {
    // a powinna
    int pos = keys.indexOf(stat);
    switch (te.getSecond()) {
    case ABSTRACT:
    inAbstracts[pos] = true;
    break;
    case TITLE:
    inTitles[pos] = true;
    break;
    }
    }
    }
    }


    }

    }


    }
    }


    if (hasAbstarct) {
    totalWithAbstact++;
    }
    if (hasTitle) {
    totalWithTitles++;
    }
    for (int j = 0; j < keys.size(); j++) {
    KeyPhraseStats stat = keys.get(j);
    totalKeyowordsInstances++;
    stat.appears++;
    boolean inAbstract = false;
    boolean inTitle = false;
    for (Pair<String, TextType> te : textsTexts) {
    if (te.getFirst().contains(stat.lowerCased)) {
    switch (te.getSecond()) {
    case ABSTRACT:
    inAbstract = true;
    break;
    case TITLE:
    inTitle = true;
    break;
    }

    }
    for (String stemmed : stat.stemmed) {
    if (te.getFirst().contains(stemmed)) {
    switch (te.getSecond()) {
    case ABSTRACT:
    inAbstract = true;
    break;
    case TITLE:
    inTitle = true;
    break;
    }



    }
    }

    }
    inAbstract = inAbstract || inAbstracts[j];
    inTitle = inTitle || inTitles[j];
    if (hasAbstarct) {
    stat.hasAbstact++;
    }

    if (inAbstract) {
    stat.appersInAbstract++;
    totalKeyowordsInstancesInAbstracts++;
    }
    if (inTitle) {
    stat.appearsInTitle++;
    totalKeyowordsInstancesInTitles++;
    }
    if (inTitle || inAbstract) {
    totalKeyowordsInstancesInTitlesOrAbstract++;
    stat.appearsInAbstractOrTitle++;
    }

    }

    }*/
    public static void main(String[] args) throws FileNotFoundException {
        if (args.length != 2) {
            System.err.println("two argument - directory with files to learn, directory with files to test");
        }
        TesterAndClassifier counter = new TesterAndClassifier();
        if (args.length > 2) {
            if ("dostem".equalsIgnoreCase(args[2])) {
                counter.stemmer = new PolishStemmer();
            }
        }
        YLanguage lang=null;
        if (args.length>3){
            lang=YLanguage.byCode(args[3]);
        }
            
        PackDirToYElementIterator iter = new PackDirToYElementIterator(new File(args[0]));
        while (iter.hasNext()) {
            YElement el = iter.next();
            counter.processYElementAndLearn(el,lang);
        }

        PrintStream ps = new PrintStream(new FileOutputStream("key-stats-" + System.currentTimeMillis() + ".log"));
        //counter.printInfo(ps);
        //ps.close();
        counter.reduceTables(ps);
        PackDirToYElementIterator iter2 = new PackDirToYElementIterator(new File(args[1]));
        while (iter2.hasNext()) {
            YElement el = iter2.next();
            counter.generateKeyWordsFor(el, ps,lang);
        }
        ps.close();

        System.exit(0);

    }
}
