package pl.edu.icm.yadda.remoting.cli;

import java.io.PrintWriter;
import java.util.Properties;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.GnuParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.config.PropertyPlaceholderConfigurer;
import org.springframework.beans.factory.xml.XmlBeanFactory;
import org.springframework.core.io.ClassPathResource;

import pl.edu.icm.yadda.analysis.relations.PersonDirectoryCreator;
import pl.edu.icm.yadda.analysis.relations.weightassigment.AdaBoostWeightAssignator;

/**
 * Zerknij na PersonDirectoryCreatorRunner
 * @author pdendek
 *
 */
public class AdaBoostWeightAssignatorRunner {
    private static final Logger log = LoggerFactory.getLogger(AdaBoostWeightAssignatorRunner.class);
    
    //zerknij na: person-directory-beans.xml w lokalizacji co poniżej    
    protected static final String BUNDLE = "/pl/edu/icm/yadda/analysis/relations/adaboost-assignator-beans.xml";

    protected static final String O_HELP = "help";
    protected static final String O_STOREDIR = "store-dir";
    protected static final String O_MAXITER = "MAX";

    protected static Options defineOptions() {
        Options options = new Options();

        Option oHelp = new Option("h", O_HELP, false, "this help message");
        options.addOption(oHelp);
        Option oStoreDir = new Option("s", O_STOREDIR, true, "Sesame's native store directory");
        oStoreDir.setRequired(true);
        options.addOption(oStoreDir);
        return options;
    }

    protected static void usage(Options options) {
        HelpFormatter helpFormatter = new HelpFormatter();
        PrintWriter writer = new PrintWriter(System.out);
        helpFormatter.printUsage(writer, 80, AdaBoostWeightAssignatorRunner.class.getSimpleName(), options);
        helpFormatter.printOptions(writer, 80, options, 1, 2);
        writer.flush();
    }

    public static void main(String args[]) throws Exception {
        Options options = defineOptions();
        CommandLineParser parser = new GnuParser();
        try {
            CommandLine commandLine = parser.parse(options, args);
            if (commandLine.hasOption(O_HELP))
                throw new ParseException("");

            Properties properties = new Properties();
            properties.setProperty("storeDirectory", commandLine.getOptionValue(O_STOREDIR));

            XmlBeanFactory factory = new XmlBeanFactory(new ClassPathResource(BUNDLE));
            PropertyPlaceholderConfigurer cfg = new PropertyPlaceholderConfigurer();
            cfg.setProperties(properties);
            cfg.postProcessBeanFactory(factory);

//            PersonDirectoryCreator pdc = (PersonDirectoryCreator) factory.getBean("personDirectoryCreator");
//            log.info("Starting transfer");
//            pdc.createPersonDirectory();
//            log.info("Transfer finished");
            
            AdaBoostWeightAssignator awa = (AdaBoostWeightAssignator) factory.getBean("adaBoostWeightAssignator");
            log.info("Starting assignation");
            awa.assignWeights();
            log.info("Assignation ended");
            
        } catch (ParseException e) {
            usage(options);
        }
    }
}
