/**
 * @author tzr
 *
 */
package pl.edu.icm.yadda.remoting.cli;

import java.io.FileOutputStream;
import java.io.OutputStream;
import java.util.LinkedList;
import java.util.List;
import java.util.prefs.Preferences;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.PosixParser;
import org.springframework.context.ConfigurableApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import pl.edu.icm.yadda.service2.CatalogParamConstants;
import pl.edu.icm.yadda.tools.IExporter;
import pl.edu.icm.yadda.tools.IExporterFactory;

public class BWMetaExporter {
	
	static class InvalidOptionException extends Exception {		
	}

	public static void main(String[] args) throws Exception {
				
		Options options = new Options();
		
		options.addOption("c", true, "specify collection. If specified, only objects belonging to specified collection will be exported.");
		options.addOption("s", true, "specify repository url");
		options.addOption("t", true, "specify custom context file");
		options.addOption("i", true, "specify id of single object to export. If the id is provided " +
				"only the object, it's predecesors and ancestors are exporter.");
		options.addOption("f", true, "specify output file");
		options.addOption("g", true, "specify format (possible values are ZIP, TAR and TGZ)");
		options.addOption("h", false, "show help");
		
		try {
		
			CommandLineParser parser = new PosixParser();
			CommandLine cl = null;
			try {			
				cl = parser.parse(options, args);
			} catch (ParseException e) {
				throw new InvalidOptionException();
			}
										
			List<String> tags = new LinkedList<String>();
												
			if (cl.hasOption("c")) {
				String collection = cl.getOptionValue("c");	
				tags.add(CatalogParamConstants.TAG_COLLECTION_PREFIX + collection);
			}
								
			String contextPath = cl.hasOption("t") ? cl.getOptionValue("t") : "classpath:pl/edu/icm/yadda/remoting/cli/cli-context.xml";
			String fileName = cl.getOptionValue("f");
			String serviceRootUrl = cl.getOptionValue("s");
			
			String rootObjectId = cl.getOptionValue("i");
			
			if (cl.hasOption("h") || (!cl.hasOption("s") && (cl.hasOption("t")))) 
				throw new InvalidOptionException();
			
			String format = "ZIP";
			if (cl.hasOption("g")) {
				format = cl.getOptionValue("g");
			} else if (fileName != null) {
				//Try to guess package format
				if (fileName.toLowerCase().endsWith(".tar")) {
	        		format = "TAR";
	        	} else if (fileName.toLowerCase().endsWith(".tar.gz") || fileName.toLowerCase().endsWith("tgz")) {
	        		format = "TGZ";
	        	} 
			}						
			
			Preferences.userRoot().put("serviceRootUrl", serviceRootUrl);
			
	        ConfigurableApplicationContext context = 
	        	new ClassPathXmlApplicationContext(contextPath);
		        
		    try {
		        
		        IExporterFactory exporterFactory = (IExporterFactory)context.getBean("exporterFactory");
		        
		        OutputStream os = fileName != null ? new FileOutputStream(fileName) : System.out;
		        
		        IExporter exporter = rootObjectId != null ? 
		        		exporterFactory.build(os, format, rootObjectId, true) :
		        			exporterFactory.build(os, format, true);
		        	        	        
		        exporter.process();
			
			} finally {			
				context.close();		
			}
		
		} catch (InvalidOptionException e) {
			HelpFormatter formatter = new HelpFormatter();
			formatter.printHelp( "export.sh", options );
		}											
        
	}

}
