/**
 * @author tzr
 *
 */
package pl.edu.icm.yadda.remoting.cli;

import static pl.edu.icm.yadda.bwmeta.model.YConstants.EXT_PREFIX_ELEMENT;

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.prefs.Preferences;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.PosixParser;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.context.ConfigurableApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import pl.edu.icm.yadda.service2.CatalogObjectMeta;
import pl.edu.icm.yadda.service2.CatalogParamConstants;
import pl.edu.icm.yadda.service2.catalog.CatalogException;
import pl.edu.icm.yadda.service2.catalog.CountingIterator;
import pl.edu.icm.yadda.service2.catalog.ICatalogFacade;
import pl.edu.icm.yadda.service2.catalog.search.MatchCriteria;
import pl.edu.icm.yadda.service2.editor.DeleteOperation;
import pl.edu.icm.yadda.service2.editor.EditorOperation;
import pl.edu.icm.yadda.service2.editor.IEditorFacade;
import pl.edu.icm.yadda.service2.editor.IEditor.EXECUTION_MODE;
import pl.edu.icm.yadda.tools.IImporter;
import pl.edu.icm.yadda.tools.IImporterFactory;

public class BWMetaImporter {
	
	
	
	static class InvalidOptionException extends Exception {

		/**
		 * @param msg the error message
		 */
		public InvalidOptionException(String msg) {
			super(msg);
		}

		/**
		 * Creates an exception without any error message
		 */
		public InvalidOptionException() {}		
	}

	public static void main(String[] args) throws Exception {
				
		Options options = new Options();
		
		options.addOption("c", true, "specify collection (with or without bwmeta1.element.element-collection- prefix)");
		options.addOption(CliParameters.CLI_SHORT_PARAM_SERV_DESCR_LOC, true, "specify repository descriptor location");
		options.addOption(CliParameters.CLI_SHORT_PARAM_CTX_FILE, true, "specify custom context file");
		options.addOption("l", true, "specify licenses (colon separated if multiple)");
		options.addOption("o", true, "specify overwrite mode (ALWAYS (default), NEVER, IF_OLDER)");
		options.addOption("r", "remove", false, "remove old data from the collection (requires -c, forbids -o)");
		options.addOption("f", true, "specify input file");
		options.addOption("g", true, "specify format (possible values are ZIP, TAR and TGZ)");
		options.addOption(CliParameters.CLI_SHORT_PARAM_HELP, false, "show help");
//		authentication related
		options.addOption(CliParameters.CLI_SHORT_PARAM_USER, "login", true, "user login to be used when importing to secured respository");
		options.addOption(CliParameters.CLI_SHORT_PARAM_PASSWD, "password", true, "user password to be used when importing to secured respository");
		options.addOption(CliParameters.CLI_SHORT_PARAM_DOMAIN, "domain", true, "user domain, optional");
        options.addOption(CliParameters.CLI_SHORT_PARAM_IPADDR, "ip", true, "ip");
//		currently not used, IServiceDiscoverer does not propagate information about repository security status
//		options.addOption("F", "force-no-authn", true, "force importing for secured repository without login/password pair provided");
		
		try {
			CommandLineParser parser = new PosixParser();
			CommandLine cl = null;
			try {			
				cl = parser.parse(options, args);
			} catch (ParseException e) {
				throw new InvalidOptionException();
			}
										
			List<String> tags = new LinkedList<String>();
			
			if (cl.hasOption("l")) {
				String licenses = cl.getOptionValue("l");
				String[] splittedLicenses = StringUtils.split(licenses, ":");
				for (String licence : splittedLicenses) tags.add(CatalogParamConstants.TAG_LICENSE_PREFIX + licence);			
			}

			String collection = null;
			if (cl.hasOption("c")) {
				String col = cl.getOptionValue("c");
				if (!col.startsWith(EXT_PREFIX_ELEMENT)) col = EXT_PREFIX_ELEMENT+"element-collection-"+col;
				collection = CatalogParamConstants.TAG_COLLECTION_PREFIX + col;
				tags.add(collection);
			}
			
			String contextPath = CommonContextHelper.pickContextPath(cl);
			System.out.println("using context path: " + contextPath);
			String fileName = cl.getOptionValue("f");
			String descriptorLocation = cl.getOptionValue(CliParameters.CLI_SHORT_PARAM_SERV_DESCR_LOC);
			
			if (cl.hasOption(CliParameters.CLI_SHORT_PARAM_HELP) || 
					(!cl.hasOption(CliParameters.CLI_SHORT_PARAM_SERV_DESCR_LOC) && (cl.hasOption(CliParameters.CLI_SHORT_PARAM_CTX_FILE)))) {
				throw new InvalidOptionException();
			}
			
			String format = "ZIP";
			if (cl.hasOption("g")) {
				format = cl.getOptionValue("g");
			} else if (fileName != null) {
				//Try to guess package format
				if (fileName.toLowerCase().endsWith(".tar")) {
	        		format = "TAR";
	        	} else if (fileName.toLowerCase().endsWith(".tar.gz") || fileName.toLowerCase().endsWith("tgz")) {
	        		format = "TGZ";
	        	} 
			}
			
			IImporter.OverwriteMode overwriteMode = IImporter.OverwriteMode.ALWAYS;
			
			if (cl.hasOption("o")) {			
				try {
					overwriteMode = IImporter.OverwriteMode.valueOf(cl.getOptionValue("o"));				
				} catch (Exception e) {
					throw new InvalidOptionException();
				}
			}

			Date cleanBefore = null;
			if (cl.hasOption("r")) {
				if (collection != null && overwriteMode == IImporter.OverwriteMode.ALWAYS)
					cleanBefore = new Date();
				else
					throw new InvalidOptionException("--remove requires overwrite mode = ALWAYS and a specified collection");
			}

			Preferences.userRoot().put("descriptorLocation", descriptorLocation);
	        ConfigurableApplicationContext context = 
	        	new ClassPathXmlApplicationContext(contextPath);

	        try {
//	        	session handling
	        	if (!CommonContextHelper.maintainSession("import", cl, context, true)) {
	        		System.exit(1);
	        	}
//		    	import block
		        IImporterFactory importerFactory = (IImporterFactory)context.getBean(
		        		CommonContextHelper.BEAN_IMPORTER_FACTORY);
		        InputStream is = fileName != null ? new FileInputStream(fileName) : System.in;
		        IImporter importer = importerFactory.build(is, true, overwriteMode, false, 
		        		tags.toArray(new String[tags.size()]), format);
		        importer.process();
		        if (cleanBefore != null) cleanup(context, cleanBefore, collection);
			} finally {			
				context.close();		
			}
		
		} catch (InvalidOptionException e) {
			if (e.getMessage() != null)	System.err.println("Error: "+e.getMessage());
			HelpFormatter formatter = new HelpFormatter();
			formatter.printHelp( "import.sh", options );
			System.exit(1);
		}											
        
	}
	
	private static int batchSize = 1000;

	/**
	 * Removes from a catalog all the objects which were not modified after a specific date
	 * and which are tagged with a specific tag.
	 *
	 * @param bf bean factory used to find the <code>CatalogFacade</code> and the <code>EditorFacade</code> beans
	 * @param cleanBefore objects modified after this date are not removed
	 * @param tag only objects with this tag are removed
	 * @throws CatalogException on catalog/editor operation errors
	 */
	private static void cleanup(BeanFactory bf, Date cleanBefore, String tag) throws CatalogException {
		System.out.print("Cleanup");
		ICatalogFacade<?> cf = (ICatalogFacade<?>) bf.getBean("CatalogFacade");
		IEditorFacade<?> ef = (IEditorFacade<?>) bf.getBean("EditorFacade");

		CountingIterator<CatalogObjectMeta> it = cf.findObjects(new MatchCriteria().until(cleanBefore).tags(tag));
		while (it.hasNext()) {
			System.out.print('.');
			LinkedList<EditorOperation> ops = new LinkedList<EditorOperation>();
			while (it.hasNext() && ops.size() < batchSize) {
				CatalogObjectMeta o = it.next();
				if (!o.getStatus().isDeleted()) {
					DeleteOperation op = new DeleteOperation();
					op.setObject(o.getId());
					ops.add(op);
				}
			}
			ef.batch(ops, EXECUTION_MODE.TRANSACTIONAL);
		}
		System.out.println(" finished");
	}
}
