/**
 * @author tzr
 *
 */
package pl.edu.icm.yadda.remoting.cli;

import java.security.InvalidParameterException;
import java.util.prefs.Preferences;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.PosixParser;
import org.springframework.context.ConfigurableApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import pl.edu.icm.yadda.service2.process.IProcessManager;
import pl.edu.icm.yadda.service2.process.Process;

public class KillProcess {

	/** 
	 * Kills process working on Yadda-repository
	 * @param args
	 */
	public static void main(String[] args) throws Exception {
        
		Options options = new Options();
		options.addOption(CliParameters.CLI_SHORT_PARAM_SERV_DESCR_LOC, true, "specify repository descriptor location, required");
		options.addOption(CliParameters.CLI_SHORT_PARAM_CTX_FILE, true, "specify custom context file, optional");
		options.addOption(CliParameters.CLI_SHORT_PARAM_PID, true, "identifier of the process to be killed, required");
		options.addOption(CliParameters.CLI_SHORT_PARAM_USER, "login", true, "user login to be used when authenticating to secured respository");
		options.addOption(CliParameters.CLI_SHORT_PARAM_PASSWD, "password", true, "user password to be used when authenticating to secured respository");
		options.addOption(CliParameters.CLI_SHORT_PARAM_DOMAIN, "domain", true, "user domain, optional");
		options.addOption(CliParameters.CLI_SHORT_PARAM_HELP, false, "show help");
		
		CommandLineParser parser = new PosixParser();
		CommandLine cli = null;
		ConfigurableApplicationContext context = null;
		String pId = null;
		try {			
			cli = parser.parse(options, args);
			
			if (!cli.getArgList().isEmpty()) {
			    System.err.println("Unexpected parameters: " + cli.getArgList());
			    System.exit(1);
			}
			
			String contextPath = CommonContextHelper.pickContextPath(cli);
			System.out.println("using context path: " + contextPath);
			
			
			if (!cli.hasOption(CliParameters.CLI_SHORT_PARAM_SERV_DESCR_LOC) ||
					!cli.hasOption(CliParameters.CLI_SHORT_PARAM_PID) ||
					cli.hasOption(CliParameters.CLI_SHORT_PARAM_HELP)) {
				HelpFormatter formatter = new HelpFormatter();
				formatter.printHelp("kill.sh", options);
				return;
			}
			
			String descriptorLocation = cli.getOptionValue(CliParameters.CLI_SHORT_PARAM_SERV_DESCR_LOC);
			Preferences.userRoot().put("descriptorLocation", descriptorLocation);
			pId = cli.getOptionValue(CliParameters.CLI_SHORT_PARAM_PID);
			context = new ClassPathXmlApplicationContext(contextPath);
			
//			maintains session
			if (!CommonContextHelper.maintainSession("kill-process", cli, context, false)) {
        		System.exit(1);
        	}
	    	
//	    	process block  
	    	IProcessManager pm = (IProcessManager)context.getBean(
	    			CommonContextHelper.PROCESS_MANAGER_FACADE);
	    	for (Process p : pm.processes()) {
				if (pId.equals(p.getId().toString())) {
					System.out.println("Waiting for the process " + p.getId() + " to be cancelled");
					p.cancel();
					System.out.println("Process canceled");
					return;
				}
			}       
			System.out.println("Process with id " + pId + " not found");
			
		} catch (ParseException e) {
			throw new InvalidParameterException(e.getMessage());
		} finally {
			if (context!=null) {
				context.close();
			}
		}
	}

}
