package pl.edu.icm.yadda.remoting.cli;

import java.io.PrintWriter;
import java.util.Properties;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.GnuParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.config.PropertyPlaceholderConfigurer;
import org.springframework.beans.factory.xml.XmlBeanFactory;
import org.springframework.core.io.ClassPathResource;

import pl.edu.icm.yadda.tools.relations.CatalogToSesame;

public class RunCatalogToSesame {
	private static final Logger log = LoggerFactory.getLogger(RunCatalogToSesame.class);

	protected static final String BUNDLE = "/pl/edu/icm/yadda/tools/relations/catalog-to-sesame-bundle.xml";

	protected static final String O_DESCR = "descr";
	protected static final String O_STOREDIR = "store-dir";
	protected static final String O_HELP = "help";
	protected static final String O_IPADDR = "ip-addr";
	protected static final String O_USER = "user";
	protected static final String O_PASS = "pass";

	protected static Options defineOptions() {
		Options options = new Options();

		Option oDescr = new Option("d", O_DESCR, true, "repository descriptor");
		oDescr.setRequired(true);
		options.addOption(oDescr);
		Option oHelp = new Option("h", O_HELP, false, "this help message");
		options.addOption(oHelp);
		Option oIpAddr = new Option("i", O_IPADDR, true, "IP address to show to AAS (only for secured repos)");
		options.addOption(oIpAddr);
		Option oStoreDir = new Option("s", O_STOREDIR, true, "Sesame's native store directory");
		oStoreDir.setRequired(true);
		options.addOption(oStoreDir);
		Option oPass = new Option("p", O_PASS, true, "password to show to AAS (only for secured repos)");
		options.addOption(oPass);
		Option oUser = new Option("u", O_USER, true, "user name to show to AAS (only for secured repos)");
		options.addOption(oUser);
		return options;
	}

	protected static void usage(Options options) {
		HelpFormatter helpFormatter = new HelpFormatter(); 
		PrintWriter writer = new PrintWriter(System.out);
		helpFormatter.printUsage(writer, 80, RunCatalogToSesame.class.getSimpleName(), options);
		helpFormatter.printOptions(writer, 80, options, 1, 2);
		writer.flush();
	}

	public static void main(String args[]) throws Exception {
		Options options = defineOptions();
		CommandLineParser parser = new GnuParser();
		try {
			CommandLine commandLine = parser.parse(options, args);
			if (commandLine.hasOption(O_HELP))
				throw new ParseException("");

			Properties properties = new Properties();
			String ipAddr = commandLine.getOptionValue(O_IPADDR);
			String user = commandLine.getOptionValue(O_USER);
			String pass = commandLine.getOptionValue(O_PASS);
			properties.setProperty("user", user == null ? "" : user);
			properties.setProperty("password", pass == null ? "" : pass);
			properties.setProperty("ipAddress", ipAddr == null ? "" : ipAddr);
			properties.setProperty("maybeLogin", ipAddr == null ? "noLogin" : "serviceLogin");
			properties.setProperty("descriptor", commandLine.getOptionValue(O_DESCR));
			properties.setProperty("storeDirectory", commandLine.getOptionValue(O_STOREDIR));

			XmlBeanFactory factory = new XmlBeanFactory(new ClassPathResource(BUNDLE));
			PropertyPlaceholderConfigurer cfg = new PropertyPlaceholderConfigurer();
			cfg.setProperties(properties);
			cfg.postProcessBeanFactory(factory);

			CatalogToSesame cts = (CatalogToSesame) factory.getBean("catalogToSesame");
			log.info("Starting transfer");
			cts.transfer();
			log.info("Transfer finished");
		} catch (ParseException e) {
			usage(options);
		}
	}
}
