package pl.edu.icm.yadda.remoting.cli;

import java.io.PrintWriter;
import java.util.Properties;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.GnuParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.config.PropertyPlaceholderConfigurer;
import org.springframework.beans.factory.xml.XmlBeanFactory;
import org.springframework.core.io.ClassPathResource;

import pl.edu.icm.yadda.analysis.relations.PersonDirectoryCreator;
import pl.edu.icm.yadda.analysis.relations.SesameRepositoryManipulator;
import pl.edu.icm.yadda.analysis.relations.test.KimToYExportableRunner;

public class RunKimToSesameImporter {
    private static final Logger log = LoggerFactory.getLogger(RunKimToSesameImporter.class);
    
    protected static final String BUNDLE = "/pl/edu/icm/yadda/analysis/relations/kim-to-sesame-beans.xml";

    protected static final String O_HELP = "help";
    protected static final String O_REP = "repoFilePath";
    protected static final String O_FIL = "inputFilePath";

    protected static final String HLP_MSG = "" +
//    		"to delete predicate give \"d\" argument \n" +
//    		"to change predicate name give \"o\" and \"n\" predicate for old and new predicate name\n" +
//    		"to compare occurence of tested predicate and pattern predicate give \"test\" and \"true\" predicate for tested predicate and pattern(true) predicate name\n" +
//    		"\n" +
//    		"in case of not setting one of argument method will not be called" +
//    		"\n" +
//    		"in case of giving arguments for more then one method\n" +
//    		"there will be execured only one method according to following priority:\n" +
//    		"1) delete method has highest priority and will be executed as the only one\n" +
//    		"2) replace method in second in priority\n" +
//    		"3) test method has the lowest priority\n" +
//    		"\n" +
//    		"test method will be followed be accuracy raport" +
    		"";
    
    protected static Options defineOptions() {
        Options options = new Options();

        Option oHelp = new Option("h", O_HELP, false, HLP_MSG);
        options.addOption(oHelp);
        
        Option oStoreDir = new Option("r", O_REP, true, "New Sesame's native store directory path");
        oStoreDir.setRequired(true);
        options.addOption(oStoreDir);
        
        Option oKimFil = new Option("k", O_FIL, true, "Kim TXT repository");
        oStoreDir.setRequired(true);
        options.addOption(oKimFil);
                
        return options;
    }

    protected static void usage(Options options) {
        HelpFormatter helpFormatter = new HelpFormatter();
        PrintWriter writer = new PrintWriter(System.out);
        helpFormatter.printUsage(writer, 80, RunKimToSesameImporter.class.getSimpleName(), options);
        helpFormatter.printOptions(writer, 80, options, 1, 2);
        writer.flush();
    }

    public static void main(String args[]) throws Exception {
        Options options = defineOptions();
        CommandLineParser parser = new GnuParser();
        try {
            CommandLine commandLine = parser.parse(options, args);
            if (commandLine.hasOption(O_HELP)){
            	log.debug(HLP_MSG);
            	throw new ParseException("");
            }
            	
            String s_rep = commandLine.getOptionValue(O_REP);
            String s_fil = commandLine.getOptionValue(O_FIL);
            
            Properties properties = new Properties();
            properties.setProperty("repoFilePath", s_rep);
            properties.setProperty("inputFilePath", s_fil);

            XmlBeanFactory factory = new XmlBeanFactory(new ClassPathResource(BUNDLE));
            PropertyPlaceholderConfigurer cfg = new PropertyPlaceholderConfigurer();
            cfg.setProperties(properties);
            cfg.postProcessBeanFactory(factory);

            log.debug("obtaining importer...");
            KimToYExportableRunner run = (KimToYExportableRunner) factory.getBean("importer");
            log.debug("obtaining importer - OK");
            
            log.debug("starting import");
            run.proceed();
            log.debug("import finished");
            
        } catch (ParseException e) {
            usage(options);
        }
    }
}
