package pl.edu.icm.yadda.remoting.cli;

import java.io.PrintWriter;
import java.util.Properties;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.GnuParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.config.PropertyPlaceholderConfigurer;
import org.springframework.beans.factory.xml.XmlBeanFactory;
import org.springframework.core.io.ClassPathResource;

import pl.edu.icm.yadda.analysis.relations.PersonDirectoryCreator;
import pl.edu.icm.yadda.analysis.relations.SesameRepositoryManipulator;

public class RunSesamePersonManipulator {
    private static final Logger log = LoggerFactory.getLogger(RunSesamePersonManipulator.class);
    
    protected static final String BUNDLE = "/pl/edu/icm/yadda/analysis/relations/person-directory-manipulator-beans.xml";

    protected static final String O_HELP = "help";
    protected static final String O_STOREDIR = "store-dir";
    protected static final String O_NEW = "newName";
    protected static final String O_OLD = "oldName";
    protected static final String O_DEL = "deleteName";
    protected static final String O_TEST = "testedName";
    protected static final String O_TRUE = "trueName";

    protected static final String HLP_MSG = "" +
    		"to delete predicate give \"d\" argument \n" +
    		"to change predicate name give \"o\" and \"n\" predicate for old and new predicate name\n" +
    		"to compare occurence of tested predicate and pattern predicate give \"test\" and \"true\" predicate for tested predicate and pattern(true) predicate name\n" +
    		"\n" +
    		"in case of not setting one of argument method will not be called" +
    		"\n" +
    		"in case of giving arguments for more then one method\n" +
    		"there will be execured only one method according to following priority:\n" +
    		"1) delete method has highest priority and will be executed as the only one\n" +
    		"2) replace method in second in priority\n" +
    		"3) test method has the lowest priority\n" +
    		"\n" +
    		"test method will be followed be accuracy raport";
    
    protected static Options defineOptions() {
        Options options = new Options();

        Option oHelp = new Option("h", O_HELP, false, HLP_MSG);
        options.addOption(oHelp);
        
        Option oStoreDir = new Option("s", O_STOREDIR, true, "Sesame's native store directory");
        oStoreDir.setRequired(true);
        options.addOption(oStoreDir);
        
        Option oNew = new Option("n", O_NEW, false, "predicate name which will replace old one");
        options.addOption(oNew);
        Option oOld = new Option("o", O_OLD, false, "predicate name which will be replaced");
        options.addOption(oOld);
        
        Option oDel = new Option("d", O_DEL, false, "predicate name which will be deleted");
        options.addOption(oDel);
        
        Option oTested = new Option("test", O_TEST, false, "predicate name which occurence will be compared with true-predicate");
        options.addOption(oTested);
        Option oTrue = new Option("true", O_TRUE, false, "predicate name which occurence will be pattern to test-predicate");
        options.addOption(oTrue);
        
        
        return options;
    }

    protected static void usage(Options options) {
        HelpFormatter helpFormatter = new HelpFormatter();
        PrintWriter writer = new PrintWriter(System.out);
        helpFormatter.printUsage(writer, 80, RunSesamePersonManipulator.class.getSimpleName(), options);
        helpFormatter.printOptions(writer, 80, options, 1, 2);
        writer.flush();
    }

    public static void main(String args[]) throws Exception {
        Options options = defineOptions();
        CommandLineParser parser = new GnuParser();
        try {
            CommandLine commandLine = parser.parse(options, args);
            if (commandLine.hasOption(O_HELP)){
            	log.debug(HLP_MSG);
            	throw new ParseException("");
            }
            	
            String s_sto = commandLine.getOptionValue(O_STOREDIR);
            String s_old = commandLine.getOptionValue(O_OLD);
            String s_new = commandLine.getOptionValue(O_NEW);
            String s_del = commandLine.getOptionValue(O_DEL);
            String s_tru = commandLine.getOptionValue(O_TRUE);
            String s_tst = commandLine.getOptionValue(O_TEST);
            
            Properties properties = new Properties();
            properties.setProperty("storeDirectory", s_sto);
            if(s_old!=null) properties.setProperty("oldName", s_old);
            else properties.setProperty("oldName", "");
            
            if(s_new!=null) properties.setProperty("newName", s_new);
            else properties.setProperty("newName", "");
            
            if(s_tru!=null) properties.setProperty("trueName", s_tru);
            else properties.setProperty("trueName", "");
            
            if(s_tst!=null) properties.setProperty("testedName", s_tst);
            else properties.setProperty("testedName", "");
            
            if(s_del!=null) properties.setProperty("deleteName", s_del);
            else properties.setProperty("deleteName", "");

            XmlBeanFactory factory = new XmlBeanFactory(new ClassPathResource(BUNDLE));
            PropertyPlaceholderConfigurer cfg = new PropertyPlaceholderConfigurer();
            cfg.setProperties(properties);
            cfg.postProcessBeanFactory(factory);

            log.debug("obtaining manipulator...");
            SesameRepositoryManipulator srm = (SesameRepositoryManipulator) factory.getBean("sesameRepositoryManipulator");
            log.debug("obtaining manipulator - OK!");
            
            log.debug("selecting method...");
            if(s_del!=null){
            	log.debug("selecting method - \"deleteRelationName\"");
            	srm.deleteRelationName();
            }else if(s_old!=null && s_new!=null){
            	log.debug("selecting method - \"changeRelationName\"");
            	srm.changeRelationName();
            }else if(s_tru!=null && s_tst!=null){
            	//at the beginning printAccuracyStats call method sameOccurenceCheck
            	//so it's ok
            	log.debug("selecting method - \"sameOccurenceCheck\" & \"printAccuracyStats\"");
            	srm.printAccuracyStats();
            }else{
            	log.debug("selecting method - FAILED");
            	log.debug("non of methods get all required parameters");
            	log.debug("exiting from method");
            }
        } catch (ParseException e) {
            usage(options);
        }
    }
}
