package pl.edu.icm.yadda.remoting.cli.service3.process;

import java.security.InvalidParameterException;
import java.util.Formatter;
import java.util.prefs.Preferences;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.PosixParser;
import org.springframework.context.ConfigurableApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import pl.edu.icm.yadda.remoting.cli.CliParameters;
import pl.edu.icm.yadda.remoting.cli.CommonContextHelper;
import pl.edu.icm.yadda.service3.process.ProcessFacade;
import pl.edu.icm.yadda.service3.process.protocol.ListProcessesEntry;

/**
 * Lists all running processes.
 * @author mhorst
 *
 */
public class ListProcesses {

	/**
	 * Lists processes currently running on the specified repository
	 * @param args
	 */
	public static void main(String[] args) throws Exception {

		Options options = new Options();
		options.addOption(CliParameters.CLI_SHORT_PARAM_SERV_DESCR_LOC, true, "specify repository descriptor location, required");
		options.addOption(CliParameters.CLI_SHORT_PARAM_CTX_FILE, true, "specify custom context file, optional");
		options.addOption(CliParameters.CLI_SHORT_PARAM_USER, "login", true, "user login to be used when authenticating to secured respository");
		options.addOption(CliParameters.CLI_SHORT_PARAM_PASSWD, "password", true, "user password to be used when authenticating to secured respository");
		options.addOption(CliParameters.CLI_SHORT_PARAM_DOMAIN, "domain", true, "user domain, optional");
		options.addOption(CliParameters.CLI_SHORT_PARAM_HELP, false, "show help");
		
		CommandLineParser parser = new PosixParser();
		CommandLine cli = null;
		ConfigurableApplicationContext context = null;
		try {			
			cli = parser.parse(options, args);
			
			if (!cli.getArgList().isEmpty()) {
			    System.err.println("Unexpected parameters: " + cli.getArgList());
			    System.exit(1);
			}
			
			String contextPath = CommonContextHelper.pickContextPath(cli);
			System.out.println("using context path: " + contextPath);
			
			if (!cli.hasOption(CliParameters.CLI_SHORT_PARAM_SERV_DESCR_LOC) || 
					cli.hasOption(CliParameters.CLI_SHORT_PARAM_HELP)) {
				HelpFormatter formatter = new HelpFormatter();
				formatter.printHelp("ps.sh", options);
				return;
			}
			
			String descriptorLocation = cli.getOptionValue(CliParameters.CLI_SHORT_PARAM_SERV_DESCR_LOC);
			Preferences.userRoot().put("descriptorLocation", descriptorLocation);
	        context = new ClassPathXmlApplicationContext(contextPath);
	        
//			maintains session
	        if (!CommonContextHelper.maintainSession("list-processes", cli, context, true)) {
        		System.exit(1);
        	}
	        
//	    	process block  
	        final ProcessFacade facade = (ProcessFacade) context.getBean(
					CommonContextHelper.PROCESS_MANAGER_FACADE3);
	        System.out.println(new Formatter().format("%1$-35s %2$-35s %3$s", "PID", "PROCESSOR", "STATE"));
	        for (ListProcessesEntry entry : facade.listRunningProcesses()) {
	        	System.out.println(new Formatter().format("%1$-35s %2$-35s %3$s", 
	        			entry.getProcessId(), entry.getProcessName(), entry.getStatus()));
	        }
			
		} catch (ParseException e) {
			throw new InvalidParameterException(e.getMessage());
		} finally {
			if (context!=null) {
				context.close();
			}
		}
	}

}
