/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package pl.edu.icm.yadda.remoting.cli.sesame;

import java.io.File;
import java.io.PrintWriter;
import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.GnuParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.openrdf.repository.RepositoryException;
import org.openrdf.repository.sail.SailRepository;
import org.openrdf.sail.nativerdf.NativeStore;
import org.slf4j.LoggerFactory;

import pl.edu.icm.yadda.bwmeta.serialization.BwmetaReader1;
import pl.edu.icm.yadda.common.YaddaException;
import pl.edu.icm.yadda.tools.BasicPackImporterFactory;
import pl.edu.icm.yadda.tools.IImporter;
import pl.edu.icm.yadda.tools.OSSArchive;
import pl.edu.icm.yadda.tools.relations.CatalogToSesame;
import pl.edu.icm.yadda.tools.relations.FakeFacadesBeingIterator;
import pl.edu.icm.yadda.tools.relations.SesameFeeder;

/**
 *
 * @author kura
 */
public class TransferYaddaPackToSesame {

    protected static final String O_STOREDIR = "store-dir";
    protected static final String O_FILENAME = "file-name";

    protected static Options defineOptions() {
		Options options = new Options();

		Option oStoreDir = new Option("s", O_STOREDIR, true, "Sesame's native store directory");

		oStoreDir.setRequired(true);
		options.addOption(oStoreDir);

                Option oFileName= new Option("f", O_FILENAME , true, "Name of pack file");
                oFileName.setRequired(true);
                options.addOption(oFileName);
                
		return options;
	}

    protected static void usage(Options options) {
		HelpFormatter helpFormatter = new HelpFormatter();
		PrintWriter writer = new PrintWriter(System.out);
		helpFormatter.printUsage(writer, 80, TransferYaddaPackToSesame.class.getSimpleName(), options);
		helpFormatter.printOptions(writer, 80, options, 1, 2);
		writer.flush();
	}

  public static void main(String[]args) throws YaddaException, RepositoryException {
      Options options = defineOptions();
		CommandLineParser parser = new GnuParser();
		try {
			CommandLine commandLine = parser.parse(options, args);

                        SailRepository sailRep=new SailRepository(new NativeStore(new File(commandLine.getOptionValue(O_STOREDIR))));
                        sailRep.initialize();
                        String filename=commandLine.getOptionValue(O_FILENAME);
                        SesameFeeder feeder=new SesameFeeder();
                        feeder.setRepository(sailRep);
                        CatalogToSesame catToS=new CatalogToSesame();
                        catToS.setSesameFeeder(feeder);
                        catToS.setBwmetaReader(new BwmetaReader1());
                        BasicPackImporterFactory fact=new BasicPackImporterFactory();
                        final FakeFacadesBeingIterator facad=new FakeFacadesBeingIterator("BWMETA1");
                        OSSArchive arch=new OSSArchive();
                        arch.setArchiveFacade2(facad);
                        arch.setStorageFacade2(facad);
                        fact.setArchive(arch);
                        fact.setArchiveFacade2(facad);
                        fact.setCatalogFacade(facad);
                        fact.setEditorFacade(facad);
                        
                       final IImporter importer = fact.build(filename, true, IImporter.OverwriteMode.ALWAYS, true, new String[0], filename.substring(filename.lastIndexOf('.')).toUpperCase());
                    Thread t = new Thread() {

                        @Override
                        public void run() {
                            try {
                                importer.process();
                            } catch (Exception ex) {
                                LoggerFactory.getLogger(TransferYaddaPackToSesame.class).error("Exception caught", ex);
                            }
                            try {
                                facad.finish();

                            } catch (InterruptedException ex) {
                                LoggerFactory.getLogger(TransferYaddaPackToSesame.class).error("Exception caught", ex);
                            }
                            System.out.println("Pack processing finished");
                        }
                    };
                     t.start();
                     catToS.transfer(facad);


                    } catch (ParseException e) {
			usage(options);
		}
  }


}
